"""Definition of a user representation"""


from pyrosalind import parser


class User:
    """User data holder, and wrapper around the parser API
    to get user data from username on ROSALIND

    """

    def __init__(self, name:str, solved_problems:iter):
        self._name = name
        self._solved = set(solved_problems)
        # self._badges = set()  # TODO
        # self._achievements = set()  # TODO


    @property
    def name(self): return self._name

    @property
    def solved(self): return self._solved

    @property
    def badges(self): return self._badges


    @staticmethod
    def from_name(name):
        """Parse ROSALIND directly to get all informations necessary"""
        return User(name, (problem for problem in parser.problems(name)))


    def __str__(self):
        return self.name + ' (' + str(len(self.solved)) + ' problems solved)'



"""Low level layer, parsing http://rosalind.info in order to get
information about users

"""

import os
from bs4 import BeautifulSoup
import requests
from urllib.request import urlopen
import pickle


USERNAME_URL = 'http://rosalind.info/users/{}'
FILE_CACHE = 'cache.pkl'
# FILE_CACHE = None  # comment to enable dumb cache feature


def problems(username:str) -> iter:
    """Yield problems solved by user with given username in ROSALIND"""
    if FILE_CACHE and os.path.exists(FILE_CACHE):
        with open(FILE_CACHE, 'rb') as fd:
            html = pickle.load(fd)
    else:
        html = _get_page(USERNAME_URL.format(username))
        if FILE_CACHE:
            with open(FILE_CACHE, 'wb') as fd:
                pickle.dump(html, fd)

    problems = _parse_problems(html)
    yield from (problem for problem, is_solved in problems if is_solved)


def _parse_problems(html:str) -> iter:
    """Yield pairs (problem, is_solved) from given rosalind html user page"""
    page = BeautifulSoup(html, 'html.parser')
    for elem in page.find_all(**{'class':'badge'}):
        yield elem.string, elem['class'] == ['badge', 'badge-success']


def _get_page(url:str) -> str:
    """Return raw html of given page"""
    return requests.get(url).text
